@@ -39,13 +39,40 @@ function usb_physical_port($usbbusdev) {
   return($physical_busid);
 }
 
-$sriov =  json_decode(getSriovInfoJson(true),true);
-$sriovvfs = getVfListByIommuGroup();
-$sriov_devices=parseVFvalues();
-$sriov_devices_settings=parseVFSettings();
+function  createIOMMUCheckBox($pciaddress,$vd,$iommu,$line) {
+  global $iommuinuse,$vfio_cfg_devices;
+    if ((strpos($line, 'Host bridge') === false) && (strpos($line, 'PCI bridge') === false)) {
+      echo in_array($iommu, $iommuinuse) ? '<tr hidden><td><input type="checkbox" value="" title="'._('In use by Unraid').'" disabled ' : '<tr hidden><td><input type="checkbox" hidden class="iommu'.$iommu.'" value="'.$pciaddress."|".$vd.'" ';
+      // check config file for two formats: <Domain:Bus:Device.Function>|<Vendor:Device> or just <Domain:Bus:Device.Function>
+      echo (in_array($pciaddress."|".$vd, $vfio_cfg_devices) || in_array($pciaddress, $vfio_cfg_devices)) ? " checked>" : ">";
+      echo "</td></tr>";
+    }
+}
 
 switch ($_POST['table']) {
 case 't1':
+  $sriov =  json_decode(getSriovInfoJson(true),true);
+  $sriovvfs = getVfListByIommuGroup();
+  $sriov_devices=parseVFvalues();
+  $sriov_devices_settings=parseVFSettings();
+
+  $filter =[];
+  $pciothers = ["0x00","0x05","0x06","0x07","0x08","0x09","0x0A","0x0B","0x0D","0x0E","0x0F","0x10","0x11","0x12","0x13"];
+
+  $showall = $_POST['showall'];
+  $showsriov = false;
+
+  if ($showall == "no") {
+    if ($_POST['showdisplay'] == 'no') $filter = array_merge($filter,["0x03","0x04"]);
+    if ($_POST['shownetwork'] == 'no') $filter[]="0x02";
+    if ($_POST['showstorage'] == 'no') $filter[]="0x01";
+    if ($_POST['showusb'] == 'no') $filter[]="0x0c";
+    if ($_POST['showother'] == 'no') $filter = array_merge($filter,$pciothers);
+    // Normalize filter prefixes for case-insensitive matching
+    $filter = array_map('strtolower', $filter);
+    $showsriov = ($_POST['showsriov'] == 'yes');
+  }
+
   exec('for group in $(ls /sys/kernel/iommu_groups/ -1|sort -n);do echo "IOMMU group $group";for device in $(ls -1 "/sys/kernel/iommu_groups/$group"/devices/);do echo -n $\'\t\';lspci -ns "$device"|awk \'BEGIN{ORS=" "}{print "["$3"]"}\';lspci -s "$device";done;done',$groups);
   if (empty($groups)) {
     exec('lspci -n|awk \'{print "["$3"]"}\'',$iommu);
@@ -59,6 +86,32 @@ case 't1':
     $BDF_REGEX = '/^[[:xdigit:]]{2}:[[:xdigit:]]{2}\.[[:xdigit:]]$/';
     $DBDF_PARTIAL_REGEX = '/[[:xdigit:]]{4}:[[:xdigit:]]{2}:[[:xdigit:]]{2}\.[[:xdigit:]]/';
     $vfio_cfg_devices = array ();
+
+    $lsiommu = [];
+    $current = null;
+
+    foreach ($groups as $line) {
+        $line = trim($line);
+
+        // Detect "IOMMU group X"
+        if (preg_match('/^IOMMU group\s+(\d+)/i', $line, $m)) {
+            $current = (int)$m[1];
+            $lsiommu[$current] = [];
+            continue;
+        }
+
+        // Match PCI ID block + PCI address
+        // Example: [8086:7d67] 00:02.0 ...
+        if ($current !== null && preg_match('/\[[0-9a-fA-F]{4}:[0-9a-fA-F]{4}\]\s+([0-9a-fA-F:.]+)/', $line, $m)) {
+            $bdf = $m[1]; // 00:02.0 or 0000:00:02.0
+            // If no domain, prepend 0000:
+            if (!preg_match('/^[0-9a-fA-F]{4}:/', $bdf)) {
+                $bdf = "0000:" . $bdf;
+            }
+            $lsiommu[$current][$bdf] = $line;
+        }
+    }
+    
     if (is_file("/boot/config/vfio-pci.cfg")) {
       // accepts space-separated list of <Bus:Device.Function> or <Domain:Bus:Device.Function> followed by an optional "|" and <Vendor:Device>
       // example: BIND=03:00.0 0000:03:00.0 03:00.0|8086:1533 0000:03:00.0|8086:1533
@@ -120,40 +173,85 @@ case 't1':
     }
     $lines = array_values(array_unique($lines, SORT_STRING));
     $iommuinuse = array ();
+
     foreach ($lines as $pciinuse){
       $string = exec("ls /sys/kernel/iommu_groups/*/devices/$pciinuse -1 -d");
       $string = substr($string,25,2);
       $iommuinuse[] = (strpos($string,'/')) ? strstr($string, '/', true) : $string;
     }
     exec('lsscsi -s',$lsscsi);
-    // Filter for 'removed' devices
+    // Build synthetic "Removed" group for devices no longer present
     $removedArr = array_filter($pci_device_diffs, function($entry) {
       return isset($entry['status']) && $entry['status'] === 'removed';
     });
-    foreach ($removedArr as $removedpci => $removeddata) {
-      $groups[] = "IOMMU "._("Removed");
-      $groups[] = "\tR[{$removeddata['device']['vendor_id']}:{$removeddata['device']['device_id']}] ".str_replace("0000:","",$removedpci)." ".trim($removeddata['device']['description'],"\n");
+    if (!empty($removedArr)) {
+      $removedKey = _("Removed");
+      if (!isset($lsiommu[$removedKey])) {
+        $lsiommu[$removedKey] = [];
+      }
+      foreach ($removedArr as $removedpci => $removeddata) {
+        $bdf = $removedpci;
+        if (!preg_match('/^[0-9a-fA-F]{4}:/', $bdf)) {
+          $bdf = "0000:".$bdf;
+        }
+        $class_id = "0x".$removeddata['device']['class'];
+        $class_prefix = substr(strtolower((string)$class_id), 0, 4);
+        if (in_array($class_prefix, $filter, true)) continue;
+        $line = "R[{$removeddata['device']['vendor_id']}:{$removeddata['device']['device_id']}] "
+              . str_replace("0000:","",$removedpci)." "
+              . trim($removeddata['device']['description'],"\n");
+        $lsiommu[$removedKey][$bdf] = $line;
+      }
     }
     $ackparm = "";
-    foreach ($groups as $line) {
-      if (!$line) continue;
-      if (in_array($line,$sriovvfs)) continue;
-      if ($line[0]=='I') {
-        if (isset($spacer)) echo "<tr><td colspan='2' class='thin'></td>"; else $spacer = true;
-        echo "</tr><tr><td>$line:</td><td>";
-        $iommu = substr($line, 12);
-        $append = true;
-      } else {
+ 
+    foreach ($lsiommu as $key => $group) {
+      if (empty($group)) {
+        continue;
+      }
+      $pciidcheck = array_key_first($group);
+      if (in_array($pciidcheck,$sriovvfs)) continue;
+      # Filter devices.
+
+      $iommu = $key;
+      $iommushow = true;
+      foreach ($group as $pciaddress => $line) {
+        if (!$line) continue;
+        [$class, $class_id, $name] = getPciClassNameAndId($pciaddress);
+        $class_prefix = substr(strtolower((string)$class_id), 0, 4);
+
+        # Do filter checks, but if filtered still create IOMMU check box.
         $line = preg_replace("/^\t/","",$line);
         $vd = trim(explode(" ", $line)[0], "[]");
-        $pciaddress = explode(" ", $line)[1];
+
+        if ($showsriov && !array_key_exists($pciidcheck,$sriov)) {
+          createIOMMUCheckBox($pciaddress, $vd, $iommu, $line);
+          break;
+        }
+        if ($showsriov && !in_array(substr($sriov[$pciidcheck]['class_id'],0,4),$allowedPCIClass)) {
+          createIOMMUCheckBox($pciaddress, $vd, $iommu, $line);
+          break;
+        }
+  
+        if (in_array($class_prefix, $filter, true)) {
+          createIOMMUCheckBox($pciaddress, $vd, $iommu, $line);
+          continue;
+        }
+
+        if ($iommushow) {
+          if (isset($spacer)) echo "<tr><td colspan='2' class='thin'></td>"; else $spacer = true;
+          echo "</tr><tr><td>IOMMU group $key:</td><td>";
+          $iommushow = false;
+          $append = true;
+        }
+
         $removed = $line[0]=='R' ? true : false;
         if ($removed) $line=preg_replace('/R/', '', $line, 1);
         if (preg_match($BDF_REGEX, $pciaddress)) {
           // By default lspci does not output the <Domain> when the only domain in the system is 0000. Add it back.
           $pciaddress = "0000:".$pciaddress;
         }
-        if ( in_array($pciaddress,$sriovvfs)) continue;
+        [$class, $class_id, $name] = getPciClassNameAndId($pciaddress);
         echo ($append) ? "" : "<tr><td></td><td>";
         exec("lspci -v -s $pciaddress", $outputvfio);
         if (preg_grep("/vfio-pci/i", $outputvfio)) {
@@ -172,7 +270,16 @@ case 't1':
         } else { echo "</td><td>"; }
         echo '</td><td title="';
         foreach ($outputvfio as $line2) echo htmlentities($line2,ENT_QUOTES)."&#10;";
-        echo '">',$line,'</td></tr>';
+        echo '">',$line,'</td><td>';
+        $pcispeed = getPciLinkInfo($pciaddress);
+        $speedcol = $pcispeed['speed_downgraded'] ? '<span class="orange-text">'.$pcispeed['current_speed'].'</span>' : $pcispeed['current_speed'];
+        if ($pcispeed['max_speed']) $speedcol .= '/'.$pcispeed['max_speed'].' '.$pcispeed['rate'];
+        if (isset($pcispeed['current_width'])) $current_width = 'x'.$pcispeed['current_width']; else $current_width = '';
+        $widthcol = $pcispeed['width_downgraded'] ? '<span class="orange-text">'.$current_width.'</span>' : $current_width;
+        if ($pcispeed['max_width']) $widthcol = $widthcol.'/x'.$pcispeed['max_width'];
+        echo $speedcol, '</td><td>', $widthcol;
+        if (isset($pcispeed['generation'])) echo'(Gen:'.$pcispeed['generation'].')';
+        echo '</td></tr>';
         if (array_key_exists($pciaddress,$pci_device_diffs)) {
           echo "<tr><td></td><td><td></td><td></td><td>";
           echo "<i class=\"fa fa-warning fa-fw orange-text\" title=\""._('PCI Change')."\n"._('Click to acknowledge').".\" onclick=\"ackPCI('".htmlentities($pciaddress)."','".htmlentities($pci_device_diffs[$pciaddress]['status'])."')\"></i>";
@@ -190,16 +297,60 @@ case 't1':
             echo "</td></tr>";
           }
         }
-      
+
+        switch (true) {
+          case (strpos($line, 'USB controller') !== false):
+            if (isset($isbound)) {
+              echo '<tr><td></td><td></td><td></td><td></td><td>',_('This controller is bound to vfio, connected USB devices are not visible'),'.</td></tr>';
+            } else {
+              exec('for usb_ctrl in $(find /sys/bus/usb/devices/usb* -maxdepth 0 -type l);do path="$(realpath "${usb_ctrl}")";if [[ $path == *'.$pciaddress.'* ]];then bus="$(cat "${usb_ctrl}/busnum")";lsusb -s $bus:|sort;fi;done',$getusb);
+              foreach($getusb as $usbdevice) {
+                [$bus,$id] = my_explode(':',$usbdevice);
+                $usbport = usb_physical_port($usbdevice);
+                if (strlen($usbport) > 7 ) {$usbport .= "\t"; } else { $usbport .= "\t\t"; }
+                echo "<tr><td></td><td></td><td></td><td></td><td>$bus Port $usbport",trim(substr($id,0,120)),"</td></tr>";
+              }
+              unset($getusb);
+            }
+            break;
+          case (strpos($line, 'SATA controller') !== false):
+          case (strpos($line, 'Serial Attached SCSI controller') !== false):
+          case (strpos($line, 'RAID bus controller') !== false):
+          case (strpos($line, 'SCSI storage controller') !== false):
+          case (strpos($line, 'IDE interface') !== false):
+          case (strpos($line, 'Mass storage controller') !== false):
+          case (strpos($line, 'Non-Volatile memory controller') !== false):
+            if (isset($isbound)) {
+              echo '<tr><td></td><td></td><td></td><td></td><td>',_('This controller is bound to vfio, connected drives are not visible'),'.</td></tr>';
+            } else {
+              exec('ls -al /sys/block/sd* /sys/block/hd* /sys/block/sr* /sys/block/nvme* 2>/dev/null | grep -i "'.$pciaddress.'"',$getsata);
+              foreach($getsata as $satadevice) {
+                $satadevice = substr($satadevice, strrpos($satadevice, '/', -1)+1);
+                $search = preg_grep('/'.$satadevice.'.*/', $lsscsi);
+                foreach ($search as $deviceline) {
+                  echo '<tr><td></td><td></td><td></td><td></td><td>',$deviceline,'</td></tr>';
+                }
+              }
+              unset($search);
+              unset($getsata);
+            }
+            break;
+          case (strpos($line, 'Ethernet controller') !== false):
+          case (strpos($line, 'Network controller') !== false):
+            $pciips = getIpAddressesByPci($pciaddress);
+            foreach($pciips as $network => $pciip) {
+              echo '<tr><td></td><td></td><td></td><td></td><td>',_("Network name"),": ",$network,'</td></tr>'; 
+              foreach ($pciip as $ip) echo '<tr><td></td><td></td><td></td><td></td><td> IP: ',$ip,'</td></tr>'; 
+            }
+            break;
+        }
 
         if (array_key_exists($pciaddress,$sriov) && in_array(substr($sriov[$pciaddress]['class_id'],0,4),$allowedPCIClass)) {
           echo "<tr><td></td><td></td><td></td><td></td><td>";
           echo _("SRIOV Available VFs").":{$sriov[$pciaddress]['total_vfs']}";
           $num_vfs= $sriov[$pciaddress]['num_vfs'];
           
-          if (isset($sriov_devices[$pciaddress])) 
-            $file_numvfs = $sriov_devices[$pciaddress]['vf_count'];
-          else $file_numvfs = 0;
+          if (isset($sriov_devices[$pciaddress])) $file_numvfs = $sriov_devices[$pciaddress]['vf_count'];else $file_numvfs = 0;
 
           echo '<label for="vf'.$pciaddress.'"> '._('Select number of VFs').': </label>';
           echo '<select class="narrow" name="vf'.$pciaddress.'" id="vf'.$pciaddress.'">';
@@ -210,8 +361,8 @@ case 't1':
 
           // Generate numeric options
           for ($i = 1; $i <= $sriov[$pciaddress]['total_vfs']; $i++) {
-          $selected = ($file_numvfs == $i) ? ' selected' : '';
-          echo "<option value=\"$i\"$selected>$i</option>";
+            $selected = ($file_numvfs == $i) ? ' selected' : '';
+            echo "<option value=\"$i\"$selected>$i</option>";
           }
 
           echo '</select>';
@@ -268,83 +419,62 @@ case 't1':
                 }
                 echo "</td></tr>";
               }
-            }         
-            if (isset($sriov_devices_settings[$pciaddress])) {
-                $mac = $sriov_devices_settings[$pciaddress]['mac'];
-            } else {
-                $mac = null;
-            }
-            $placeholder = empty($mac) ? _('Dynamic allocation') : '';
-            $saved_mac  = empty($mac) ? '' : htmlspecialchars($mac, ENT_QUOTES);
-            $current_mac  = empty($vrf['mac']) ? '' : htmlspecialchars(strtoupper($vrf['mac']), ENT_QUOTES);
+            }                  
             echo "<tr><td></td><td></td><td></td><td></td><td>";
-            echo '<label for="mac_address">'._("MAC Address").':</label>';
-            echo "<input class='narrow' type=\"text\" name=\"vfmac$pciaddress\" id=\"vfmac$pciaddress\" value=\"$saved_mac\" placeholder=\"$placeholder\">";
-            echo ' <a class="info" href="#" title="'._("Generate MAC").'" onclick="generateMAC(\''.htmlentities($pciaddress).'\'); return false;"><i class="fa fa-refresh mac_generate"> </i></a>';
-            echo ' <a class="info" href="#" title="'._("Save MAC config").'" onclick="saveVFSettingsConfig(\''.htmlentities($pciaddress).'\',\''.htmlentities($vd).'\'); return false;"><i class="fa fa-save"> </i></a>';
+            $class_id = substr($vrf['class_id'],0,4);
+            switch ($class_id) {   
+              case "0x02": # Network controller
+                if (isset($sriov_devices_settings[$pciaddress])) {
+                    $mac = $sriov_devices_settings[$pciaddress]['mac'];
+                } else {
+                    $mac = null;
+                }
+                $placeholder = empty($mac) ? _('Dynamic allocation') : '';
+                $saved_mac  = empty($mac) ? '' : htmlspecialchars($mac, ENT_QUOTES);
+                $current_mac  = empty($vrf['mac']) ? '' : htmlspecialchars(strtoupper($vrf['mac']), ENT_QUOTES);
+                echo '<label for="mac_address">'._("MAC Address").':</label>';
+                echo "<input class='narrow' type=\"text\" name=\"vfmac$pciaddress\" id=\"vfmac$pciaddress\" value=\"$saved_mac\" placeholder=\"$placeholder\">";
+                echo ' <a class="info" href="#" title="'._("Generate MAC").'" onclick="generateMAC(\''.htmlentities($pciaddress).'\'); return false;"><i class="fa fa-refresh mac_generate"> </i></a>';
+                $action = _("Apply now VFIO and MAC Address");
+                $saveaction = _("Save MAC and vfio config");
+                break;
+              case "0x03": # Display controller
+                echo '<span>'._($vrf['class']).'</span>';
+                $mac = null;
+                $action = _("Apply now VFIO binding");
+                $saveaction = _("Save Vfio config");
+                break;
+              default:
+            }
+            echo ' <a class="info" href="#" title="'.$saveaction.'" onclick="saveVFSettingsConfig(\''.htmlentities($pciaddress).'\',\''.htmlentities($vd).'\',\''.htmlentities($class_id).'\'); return false;"><i class="fa fa-save"> </i></a>';
             if (isset($sriov_devices_settings[$pciaddress])) {
               $filevfio = $sriov_devices_settings[$pciaddress]['vfio'] == 1 ? true : false;
             } else $filevfio = false;
             $vfiocheck = $vrf['driver'] == "vfio-pci" ? true:false;
-            echo ' <a class="info" href="#" title="'._("Action VFs update").'" onclick="applyVFSettings(\''.htmlentities($pciaddress).'\',\''.htmlentities($vd).'\',\''.htmlentities($vfiocheck).'\',\''.$current_mac.'\'); return false;"><i title="Apply now VFIO and MAC Address" class="fa fa-play"></i></a> ';
-            if ($vrf['driver'] != "vfio-pci") echo _("Current").": ";
-            echo $vrf['driver'] == "vfio-pci" ? _("Bound to VFIO") : strtoupper($vrf['mac']);
+            echo ' <a class="info" href="#" title="'._("Action VFs update").'" onclick="applyVFSettings(\''.htmlentities($pciaddress).'\',\''.htmlentities($vd).'\',\''.htmlentities($vfiocheck).'\',\''.$current_mac.'\',\''.$class_id.'\'); return false;"><i title="'.$action.'" class="fa fa-play"></i></a> ';
+            if ($class_id == "0x02") {
+              if ($vrf['driver'] != "vfio-pci" ) echo _("Current").": ";
+              echo $vrf['driver'] == "vfio-pci" ? _("Bound to VFIO") : strtoupper($vrf['mac']);
+            }
             $vfstatus ="";
             if ($filevfio != $vfiocheck || (strtoupper($vrf['mac']) != $mac && $mac != null && $vrf['mac']!=null )) $vfstatus = "<i class=\"fa fa-warning fa-fw orange-text\"></i> ".sprintf(_("Pending action or reboot"));
             echo " <span id=vfstatus$pciaddress>$vfstatus</span>";
             echo "</td></tr>";  
           }
         }
-
         unset($outputvfio);
-        switch (true) {
-          case (strpos($line, 'USB controller') !== false):
-            if (isset($isbound)) {
-              echo '<tr><td></td><td></td><td></td><td></td><td>',_('This controller is bound to vfio, connected USB devices are not visible'),'.</td></tr>';
-            } else {
-              exec('for usb_ctrl in $(find /sys/bus/usb/devices/usb* -maxdepth 0 -type l);do path="$(realpath "${usb_ctrl}")";if [[ $path == *'.$pciaddress.'* ]];then bus="$(cat "${usb_ctrl}/busnum")";lsusb -s $bus:|sort;fi;done',$getusb);
-              foreach($getusb as $usbdevice) {
-                [$bus,$id] = my_explode(':',$usbdevice);
-                $usbport = usb_physical_port($usbdevice);
-                if (strlen($usbport) > 7 ) {$usbport .= "\t"; } else { $usbport .= "\t\t"; }
-                echo "<tr><td></td><td></td><td></td><td></td><td>$bus Port $usbport",trim($id),"</td></tr>";
-              }
-              unset($getusb);
-            }
-            break;
-          case (strpos($line, 'SATA controller') !== false):
-          case (strpos($line, 'Serial Attached SCSI controller') !== false):
-          case (strpos($line, 'RAID bus controller') !== false):
-          case (strpos($line, 'SCSI storage controller') !== false):
-          case (strpos($line, 'IDE interface') !== false):
-          case (strpos($line, 'Mass storage controller') !== false):
-          case (strpos($line, 'Non-Volatile memory controller') !== false):
-            if (isset($isbound)) {
-              echo '<tr><td></td><td></td><td></td><td></td><td>',_('This controller is bound to vfio, connected drives are not visible'),'.</td></tr>';
-            } else {
-              exec('ls -al /sys/block/sd* /sys/block/hd* /sys/block/sr* /sys/block/nvme* 2>/dev/null | grep -i "'.$pciaddress.'"',$getsata);
-              foreach($getsata as $satadevice) {
-                $satadevice = substr($satadevice, strrpos($satadevice, '/', -1)+1);
-                $search = preg_grep('/'.$satadevice.'.*/', $lsscsi);
-                foreach ($search as $deviceline) {
-                  echo '<tr><td></td><td></td><td></td><td></td><td>',$deviceline,'</td></tr>';
-                }
-              }
-              unset($search);
-              unset($getsata);
-            }
-            break;
-        }
         unset($isbound);
         $append = false;
+
       }
+      $iommushow = true;
     }
     echo '<tr><td></td><td></td><td></td><td></td><td><br>';
     if (file_exists("/var/log/vfio-pci") && filesize("/var/log/vfio-pci")) {
       echo '<input id="viewlog" type="button" value="'._('View VFIO-PCI Log').'" onclick="openTerminal(\'log\',\'vfio-pci\',\'vfio-pci\')">';
     }
     if ($ackparm == "") $ackdisable =" disabled "; else $ackdisable = "";
-    echo '<input id="applycfg" type="submit" disabled value="'._('Bind selected to VFIO at Boot').'" onclick="applyCfg();" '.(isset($noiommu) ? "style=\"display:none\"" : "").'>';
+    echo '<input id="applycfg" type="submit"'.$applyhidden.' disabled value="'._('Bind selected to VFIO at Boot').'" onclick="applyCfg();" '.(isset($noiommu) ? "style=\"display:none\"" : "").'>';
     echo '<span id="warning"></span>';
     echo '<input id="applypci" type="submit"'.$ackdisable.' value="'._('Acknowledge all PCI changes').'" onclick="ackPCI(\''.htmlentities($ackparm).'\',\'all\')" >';
     echo '</td></tr>';
